/*
 *  mb8dog.c - mb800 WatchDog driver
 *
 *  Copyright (C) 2003 Grzegorz Jaskiewicz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>            
#include <linux/ioport.h>
#include <asm/system.h>             
#include <asm/io.h>                 
#include <asm/uaccess.h>            

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#else
#error "this driver requires devfs, otherwise it would not work - sorry"
#endif

#define DEBUG
#ifndef DEBUG
# define dprintk(format, args...)
#else
# define dprintk(format, args...) printk(KERN_DEBUG "mb8wdog: " , ## args)
#endif

//#define CHECK_BEFORE_USE

static devfs_handle_t fs_handle;

static int close_wdog(struct inode *inode, struct file *filp);
static int open_wdog( struct inode *inode, struct file *filp);
static ssize_t write_wdog( struct file *, const char *, size_t, loff_t*);


static struct file_operations tabfs = {
    owner   : THIS_MODULE,
    open    : open_wdog,      /* open */
    write   : write_wdog,	/* write */
    release : close_wdog,   /* release */
};

static void LockChip()
{
	outb(0xAA, 0x2E);
}

static void UnLockChip()
{
	outb(0x87, 0x2E);
	outb(0x87, 0x2E);
}

static void WriteReg(unsigned char index, unsigned char value)
{
	outb(index, 0x2E);
	outb(value, 0x2F);
}

static unsigned char ReadReg(char v)
{
	outb(v ,0x2E);
	return inb(0x2F);
}

static void EnableAndSetWatchdog(unsigned char dupa)
{
	unsigned char o;
	
	UnLockChip();
	
	o=ReadReg(0x2b);
	o=o&(~0x10);
	WriteReg(0x2b, o);

	WriteReg(7,8);

	o=ReadReg((unsigned int)0xf5);
	o=o&(~8);
	WriteReg(0xf5, o);

	WriteReg(0xf6, dupa);

	WriteReg(0x30, 1);
	UnLockChip();

}

static void DisableWatchdog()
{
	UnLockChip();
	WriteReg(7, 8);
	WriteReg(0xF6, 0);
	WriteReg(0x30, 0);
	LockChip();
}

		    

/* Initialize the module */
int init_module()
{
// it unused, becouse kernel wouldn't me take that 
// space anyway (dunno why)
// 666 
#ifdef  CHECK_BEFORE_USE
    if (check_region(0x2e, 2))
    {
	dprintk("my I/O space is allready in use, can't share it .. sorry\n");
	return -1;
    }

    request_region(0x2e, 2, "mb800 watchdog");
#endif
// 666

    fs_handle= devfs_register(NULL, "mb8wdog", DEVFS_FL_AUTO_DEVNUM,
				0,0, S_IFCHR|S_IRUGO|S_IWUGO,
				&tabfs, NULL);    
    
    DisableWatchdog();
    dprintk("module loaded\n");
    
    return 0;
}


/* Cleanup */
void cleanup_module()
{
    devfs_unregister(fs_handle);

#ifdef CHECK_BEFORE_USE
    release_region(0x2e,2);
#endif

    dprintk("module unloaded\n");
}

static ssize_t write_wdog( struct file *f, const char *buf, size_t count, loff_t *pos )
{
    unsigned char b;

    // anyway , i will read just 1st byte
    
    if (copy_from_user(&b, buf, 1))
    {
	return -EFAULT;
    }

    if (b)
    {    
	dprintk("setting watchdog\n");
	EnableAndSetWatchdog(b);
    }
    else
    {
	DisableWatchdog();
	dprintk("watchdog disabled\n");
    }
    
    
    return count;
}

// this can be actually the same function for open and close
static int open_wdog( struct inode *inode, struct file *filp)
{

    return 0;
}

static int close_wdog(struct inode *inode, struct file *filp)
{
    
    return 0;
}


MODULE_AUTHOR("Grzegorz Jaskiewicz <gj@pointblue.com.NOSPAM.pl>");
MODULE_DESCRIPTION("intel boards mb800 watchdog driver");
MODULE_LICENSE("GPL");

